@@ -1,12 +1,12 @@
 """
 aurora-data-api - A Python DB-API 2.0 client for the AWS Aurora Serverless Data API
 """
-import os, datetime, ipaddress, time, random, string, logging, itertools, reprlib, json, re
-from uuid import UUID
+import os, datetime, ipaddress, uuid, time, random, string, logging, itertools, reprlib, json, re, threading
 from decimal import Decimal
 from collections import namedtuple
+from collections.abc import Mapping
 from .exceptions import (Warning, Error, InterfaceError, DatabaseError, DataError, OperationalError, IntegrityError,
-                         InternalError, ProgrammingError, NotSupportedError)
+                         InternalError, ProgrammingError, NotSupportedError, MySQLError, PostgreSQLError)
 from .mysql_error_codes import MySQLErrorCodes
 from .postgresql_error_codes import PostgreSQLErrorCodes
 import boto3
@@ -36,15 +36,16 @@ ColumnDescription.__new__.__defaults__ = (None,) * len(ColumnDescription._fields
 
 logger = logging.getLogger(__name__)
 
-postgresql_error_reg = re.compile(r'^ERROR: (.*)[\s+]+Position: ([0-9]+); SQLState: ([0-9A-Z]+)$')
-
 
 class AuroraDataAPIClient:
+    _client_init_lock = threading.Lock()
+
     def __init__(self, dbname=None, aurora_cluster_arn=None, secret_arn=None, rds_data_client=None, charset=None,
                  continue_after_timeout=None):
         self._client = rds_data_client
         if rds_data_client is None:
-            self._client = boto3.client("rds-data")
+            with self._client_init_lock:
+                self._client = boto3.client("rds-data")
         self._dbname = dbname
         self._aurora_cluster_arn = aurora_cluster_arn or os.environ.get("AURORA_CLUSTER_ARN")
         self._secret_arn = secret_arn or os.environ.get("AURORA_SECRET_ARN")
@@ -117,13 +118,13 @@ class AuroraDataAPICursor:
         "cidr": ipaddress.ip_network,
         "date": datetime.date,
         "inet": ipaddress.ip_address,
-        "json": "json",
-        "jsonb": dict,  # TODO
-        "money": str,  # TODO
+        "json": dict,
+        "jsonb": dict,
+        "money": str,
         "text": str,
         "time": datetime.time,
         "timestamp": datetime.datetime,
-        "uuid": UUID,
+        "uuid": uuid.uuid4,
         "numeric": Decimal,
         "decimal": Decimal
     }
@@ -141,8 +142,6 @@ class AuroraDataAPICursor:
         datetime.time: "TIME",
         datetime.datetime: "TIMESTAMP",
         Decimal: "DECIMAL",
-        "json": "JSON",
-        UUID: "UUID"
     }
 
     def __init__(self, client=None, dbname=None, aurora_cluster_arn=None, secret_arn=None, transaction_id=None,
@@ -211,20 +210,30 @@ class AuroraDataAPICursor:
         return execute_args
 
     def _format_parameter_set(self, parameters):
+        if not isinstance(parameters, Mapping):
+            raise NotSupportedError("Expected a mapping of parameters. Array parameters are not supported.")
         return [self.prepare_param(k, v) for k, v in parameters.items()]
 
-    def _get_database_error(self, origin_error):
-        error_msg = getattr(origin_error, "response", {}).get("Error", {}).get("Message", "")
+    def _get_database_error(self, original_error):
+        error_msg = getattr(original_error, "response", {}).get("Error", {}).get("Message", "")
         try:
-            if error_msg.startswith("Database error code"):  # MySQL error
-                code, msg = (s.split(": ", 1)[1] for s in error_msg.split(". ", 1))
-                return DatabaseError(MySQLErrorCodes(int(code)), msg)
-            elif error_msg.startswith("ERROR: "):  # Postgresql error
-                msg, pos, code = postgresql_error_reg.match(error_msg).groups()
-                return DatabaseError(PostgreSQLErrorCodes(code), msg, pos)
+            res = re.search(r"Error code: (\d+); SQLState: (\d+)$", error_msg)
+            if res:  # MySQL error
+                error_code = int(res.group(1))
+                error_class = MySQLError.from_code(error_code)
+                error = error_class(error_msg)
+                error.response = getattr(original_error, "response", {})
+                return error
+            res = re.search(r"ERROR: .*\n  Position: (\d+); SQLState: (\w+)$", error_msg)
+            if res:  # PostgreSQL error
+                error_code = res.group(2)
+                error_class = PostgreSQLError.from_code(error_code)
+                error = error_class(error_msg)
+                error.response = getattr(original_error, "response", {})
+                return error
         except Exception:
             pass
-        return DatabaseError(origin_error)
+        return DatabaseError(original_error)
 
     def execute(self, operation, parameters=None):
         self._current_response, self._iterator, self._paging_state = None, None, None
@@ -300,10 +309,6 @@ class AuroraDataAPICursor:
             if col_desc and col_desc.type_code in self._data_api_type_hint_map:
                 if col_desc.type_code == Decimal:
                     scalar_value = Decimal(scalar_value)
-                elif col_desc.type_code == "json":
-                    scalar_value = json.loads(scalar_value)
-                elif col_desc.type_code == UUID:
-                    scalar_value = UUID(scalar_value)
                 else:
                     try:
                         scalar_value = col_desc.type_code.fromisoformat(scalar_value)
